<?php

namespace Drupal\Tests\commerce_swiss_post\Kernel;

use Drupal\commerce_shipping\Entity\Shipment;
use Drupal\commerce_shipping\Entity\ShipmentType;
use Drupal\commerce_shipping\Entity\ShippingMethod;
use Drupal\commerce_swiss_post\AccessTokenProviderInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\DrupalKernel;
use Drupal\physical\Weight;
use Drupal\physical\WeightUnit;
use Drupal\profile\Entity\Profile;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Psr7\Response;
use Prophecy\Argument;
use Symfony\Component\HttpFoundation\Request;

/**
 * @coversDefaultClass \Drupal\commerce_swiss_post\LabelGenerator
 *
 * @group commerce_swiss_post
 */
class LabelGeneratorTest extends SwissPostKernelTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'entity_reference_revisions',
    'state_machine',
    'profile',
    'commerce_number_pattern',
    'commerce_order',
    'commerce_payment',
    'commerce_shipping',
    'physical',
    'field',
    'file',
    'commerce_order',
    'commerce_swiss_post',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $this->installEntitySchema('file');
    $this->installSchema('file', 'file_usage');

    $trait_id = 'commerce_swiss_post_barcode_label';
    $shipment_type = ShipmentType::load('default');
    $shipment_type->setTraits([$trait_id]);
    $shipment_type->save();

    $trait_manager = \Drupal::service('plugin.manager.commerce_entity_trait');
    $trait = $trait_manager->createInstance($trait_id);
    $trait_manager->installTrait($trait, 'commerce_shipment', $shipment_type->id());

    $request = Request::create('/');
    $site_path = DrupalKernel::findSitePath($request);
    $this->setSetting('file_private_path', $site_path . '/private');

    $this->config('commerce_swiss_post.settings')
      ->set('franking_license', '00001111')
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container) {
    parent::register($container);

    $container->register('stream_wrapper.private', 'Drupal\Core\StreamWrapper\PrivateStream')
      ->addTag('stream_wrapper', ['scheme' => 'private']);
  }

  /**
   * @covers ::generateLabels
   * @covers ::buildRequestData
   */
  public function testPersonalLabel() {

    $access_token_provider = $this->prophesize(AccessTokenProviderInterface::class);
    $access_token_provider->getAccessToken()->willReturn('super-secure-token');

    $this->container->set('commerce_swiss_post.access_token_provider', $access_token_provider->reveal());
    $this->container->set('commerce_swiss_post.label_generator', NULL);

    $http_client = $this->prophesize(ClientInterface::class);
    $request_data = $this->getRequestData();

    $expected_options = [
      'json' => $request_data,
      'headers' => [
        'Authorization' => 'Bearer super-secure-token',
        'Accept' => 'application/json',
        'Content-Type' => 'application/json',
        ],
    ];

    $pdf_content = 'I am a Barcode PDF';
    $expected_tracking_code = '996006172900000001';
    $response_data = [
      'item' => [
        'identCode' => $expected_tracking_code,
        'label' => [
          0 => \base64_encode($pdf_content),
        ],
      ],
    ];

    $response = new Response(200, [], Json::encode($response_data));

    $http_client->request('POST', 'https://wedec.post.ch/api/barcode/v1/generateAddressLabel', Argument::is($expected_options))->willReturn($response)->shouldBeCalledTimes(1);
    $this->container->set('http_client', $http_client->reveal());

    /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $shipment */
    $shipment = Shipment::create([
      'type' => 'default',
      'state' => 'ready',
      'title' => 'Shipment',
      'order_id' => $this->order->id(),
    ]);
    $shipment->setWeight(new Weight('4.9', WeightUnit::KILOGRAM));

    /** @var \Drupal\commerce_shipping\Entity\ShippingMethodInterface $shipping_method */
    $shipping_method = ShippingMethod::create([
      'name' => $this->randomString(),
      'status' => 1,
      'plugin' => [
        'target_plugin_id' => 'swiss_post',
        'target_plugin_configuration' => [
          'shipping_method' => 'PRI',
        ],
      ],
    ]);
    $shipping_method->save();
    $shipment->setShippingMethod($shipping_method);

    /** @var \Drupal\profile\Entity\ProfileInterface $profile */
    $profile = Profile::create([
      'type' => 'customer',
      'address' => [
        'country_code' => 'CH',
        'address_line1' => 'Example Street 77',
        'postal_code' => 9999,
        'locality' => 'Example',
        'given_name' => 'Example',
        'family_name' => 'Person',
      ],
    ]);
    $profile->save();
    $shipment->setShippingProfile($profile);

    $shipment->save();

    $label_generator = $this->container->get('commerce_swiss_post.label_generator');
    $this->assertEquals($pdf_content, $label_generator->generateLabels([$shipment]));

    $shipment = Shipment::load($shipment->id());
    $this->assertEquals($expected_tracking_code, $shipment->getTrackingCode());

    // Call it a second time, ensure the API is only called once.
    $this->assertEquals($pdf_content, $label_generator->generateLabels([$shipment]));
  }

  /**
   * Test organisation label generation.
   */
  public function testOrganisationLabel() {
    $access_token_provider = $this->prophesize(AccessTokenProviderInterface::class);
    $access_token_provider->getAccessToken()->willReturn('super-secure-token');

    $this->container->set('commerce_swiss_post.access_token_provider', $access_token_provider->reveal());
    $this->container->set('commerce_swiss_post.label_generator', NULL);

    $http_client = $this->prophesize(ClientInterface::class);
    $request_data = $this->getRequestData();
    $request_data['item']['recipient']['name2'] = 'Llama Inc.';

    $expected_options = [
      'json' => $request_data,
      'headers' => [
        'Authorization' => 'Bearer super-secure-token',
        'Accept' => 'application/json',
        'Content-Type' => 'application/json',
      ],
    ];

    $pdf_content = 'I am a Barcode PDF';
    $expected_tracking_code = '996006172900000001';
    $response_data = [
      'item' => [
        'identCode' => $expected_tracking_code,
        'label' => [
          0 => \base64_encode($pdf_content),
        ],
      ],
    ];

    $response = new Response(200, [], Json::encode($response_data));

    $http_client->request('POST', 'https://wedec.post.ch/api/barcode/v1/generateAddressLabel', Argument::is($expected_options))->willReturn($response);
    $this->container->set('http_client', $http_client->reveal());

    /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $shipment */
    $shipment = Shipment::create([
      'type' => 'default',
      'state' => 'ready',
      'title' => 'Shipment',
      'order_id' => $this->order->id(),
    ]);
    $shipment->setWeight(new Weight('4.9', WeightUnit::KILOGRAM));

    /** @var \Drupal\commerce_shipping\Entity\ShippingMethodInterface $shipping_method */
    $shipping_method = ShippingMethod::create([
      'name' => $this->randomString(),
      'status' => 1,
      'plugin' => [
        'target_plugin_id' => 'swiss_post',
        'target_plugin_configuration' => [
          'shipping_method' => 'PRI',
        ],
      ],
    ]);
    $shipping_method->save();
    $shipment->setShippingMethod($shipping_method);

    /** @var \Drupal\profile\Entity\ProfileInterface $profile */
    $profile = Profile::create([
      'type' => 'customer',
      'address' => [
        'country_code' => 'CH',
        'address_line1' => 'Example Street 77',
        'postal_code' => 9999,
        'locality' => 'Example',
        'organization' => 'Llama Inc.',
        'given_name' => 'Example',
        'family_name' => 'Person',
      ],
    ]);
    $profile->save();
    $shipment->setShippingProfile($profile);

    $shipment->save();

    $label_generator = $this->container->get('commerce_swiss_post.label_generator');
    $this->assertEquals($pdf_content, $label_generator->generateLabels([$shipment]));

    // Assert that the file was created.
    $file_uri = $shipment->get('commerce_swisspost_barcode_label')->entity->getFileUri();
    $this->assertEquals($pdf_content, \file_get_contents($file_uri));
  }

  /**
   * @covers ::generateLabels
   * @covers ::buildRequestData
   */
  public function testPrintPreview() {
    $access_token_provider = $this->prophesize(AccessTokenProviderInterface::class);
    $access_token_provider->getAccessToken()->willReturn('super-secure-token');

    $this->config('commerce_swiss_post.settings')
      ->set('print_preview', TRUE)
      ->save();

    $this->container->set('commerce_swiss_post.access_token_provider', $access_token_provider->reveal());
    $this->container->set('commerce_swiss_post.label_generator', NULL);

    $http_client = $this->prophesize(ClientInterface::class);
    $request_data = $this->getRequestData();
    $request_data['labelDefinition']['printPreview'] = TRUE;

    $expected_options = [
      'json' => $request_data,
      'headers' => [
        'Authorization' => 'Bearer super-secure-token',
        'Accept' => 'application/json',
        'Content-Type' => 'application/json',
      ],
    ];

    $pdf_content = 'I am a Barcode PDF';
    $expected_tracking_code = '996006172900000001';
    $response_data = [
      'item' => [
        'identCode' => $expected_tracking_code,
        'label' => [
          0 => \base64_encode($pdf_content),
        ],
      ],
    ];

    $response = new Response(200, [], Json::encode($response_data));

    $http_client->request('POST', 'https://wedec.post.ch/api/barcode/v1/generateAddressLabel', Argument::exact($expected_options))->willReturn($response);
    $this->container->set('http_client', $http_client->reveal());
  }

  /**
   * @covers ::generateLabels
   * @covers ::buildRequestData
   */
  public function testMissingShippingMethodConfiguration() {

    $access_token_provider = $this->prophesize(AccessTokenProviderInterface::class);
    $access_token_provider->getAccessToken()->willReturn('super-secure-token');
    $this->container->set('commerce_swiss_post.access_token_provider', $access_token_provider->reveal());
    $this->container->set('commerce_swiss_post.label_generator', NULL);

    /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $shipment */
    $shipment = Shipment::create([
      'type' => 'default',
      'state' => 'ready',
      'title' => 'Shipment',
      'order_id' => $this->order->id(),
    ]);
    $shipment->setWeight(new Weight('4.9', WeightUnit::KILOGRAM));

    /** @var \Drupal\commerce_shipping\Entity\ShippingMethodInterface $shipping_method */
    $shipping_method = ShippingMethod::create([
      'name' => 'Example Method',
      'status' => 1,
      'plugin' => [
        'target_plugin_id' => 'swiss_post',
        'target_plugin_configuration' => [
          'shipping_method' => '',
        ],
      ],
    ]);
    $shipping_method->save();
    $shipment->setShippingMethod($shipping_method);

    /** @var \Drupal\profile\Entity\ProfileInterface $profile */
    $profile = Profile::create([
      'type' => 'customer',
      'address' => [
        'country_code' => 'CH',
        'address_line1' => 'Example Street 77',
        'postal_code' => 9999,
        'locality' => 'Example',
        'given_name' => 'Example',
        'family_name' => 'Person',
      ],
    ]);
    $profile->save();
    $shipment->setShippingProfile($profile);

    $shipment->save();

    $this->expectException(\RuntimeException::class);
    $this->expectExceptionMessage('Shipping method Example Method is not configured');

    $label_generator = $this->container->get('commerce_swiss_post.label_generator');
    $label_generator->generateLabels([$shipment]);
  }

  /**
   * Returns example expected request data.
   *
   * @return array
   *   The expected request data.
   */
  public function getRequestData() {
    return [
      'language' => 'EN',
      'frankingLicense' => '00001111',
      'ppFranking' => FALSE,

      'customer' => [
        'name1' => 'Default store',
        'name2' => NULL,
        'street' => 'Store street',
        'zip' => '8048',
        'city' => 'Zürich',
        'country' => 'CH',
        'logo' => NULL,
        'logoFormat' => NULL,
        'logoRotation' => 0,
        'logoAspectRatio' => NULL,
        'logoHorizontalAlign' => NULL,
        'logoVerticalAlign' => NULL,
        'domicilePostOffice' => NULL,
        'pobox' => NULL,
      ],
      'customerSystem' => NULL,
      'labelDefinition' => [
        'labelLayout' => 'A6',
        'printAddresses' => 'RECIPIENT_AND_CUSTOMER',
        'imageFileType' => 'PDF',
        'imageResolution' => 300,
        'printPreview' => FALSE,
      ],
      'sendingID' => NULL,
      'item' => [
        'notification' => [],
        'additionalData' => [],
        'itemID' => NULL,
        'itemNumber' => NULL,
        'recipient' => [
          'postIdent' => NULL,
          'title' => NULL,
          'personallyAddressed' => FALSE,
          'name1' => 'Person',
          'firstName' => 'Example',
          'name2' => NULL,
          'name3' => NULL,
          'addressSuffix' => NULL,
          'street' => 'Example Street',
          'houseNo' => '77',
          'floorNo' => NULL,
          'mailboxNo' => NULL,
          'zip' => '9999',
          'city' => 'Example',
          'country' => 'CH',
          'phone' => NULL,
          'mobile' => NULL,
          'labelAddress' => NULL,
          'pobox' => NULL,
          'email' => NULL,
        ],
        'attributes' => [
          'przl' => [
            0 => 'PRI',
          ],
          'freeText' => NULL,
          'deliveryDate' => NULL,
          'parcelNo' => 0,
          'parcelTotal' => 0,
          'deliveryPlace' => NULL,
          'proClima' => FALSE,
          'weight' => '4900',
          'unnumbers' => NULL,
        ],
      ],
    ];
  }

}
